/*
* TeamSpeak 3 server Hello Whispers sample
*
* Copyright (c) 2007-2017 TeamSpeak-Systems
*/
#include "common.h"

#include <teamspeak/public_definitions.h>
#include <teamspeak/serverlib.h>
#include <teamspeak/serverlib_publicdefinitions.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

bool is_terminating = false;
bool is_whispering = false;

void toggle_whisper(uint64 server_id)
{
    anyID* all_clients_online;
    CHECK_ERROR_FATAL(ts3server_getClientList(server_id, &all_clients_online));

    if (is_whispering == false)
    {
        if (all_clients_online[0] != 0)
        {
            uint64* all_channels;
            if (CHECK_ERROR(ts3server_getChannelList(server_id, &all_channels)))
            {
                for (int i = 0; all_clients_online[i] != 0; ++i)
                {
                    CHECK_ERROR(ts3server_setClientWhisperList(server_id, all_clients_online[i], all_channels, NULL));
                }
                ts3server_freeMemory(all_channels);
            }
        }
    }
    else
    {
        for (int i = 0; all_clients_online[i] != 0; ++i)
        {
            CHECK_ERROR(ts3server_setClientWhisperList(server_id, all_clients_online[i], NULL, NULL));
        }
    }

    ts3server_freeMemory(all_clients_online);
    is_whispering = !is_whispering;
    printf("Whispering mode %s\n", is_whispering ? "on." : "off.");
}

uint64 create_channel(uint64 server_id, const char* channel_name)
{
    /* Create Channel Part */
    uint64 channel_parent_id = 0;
    uint64 channel_id;

    /* Set data of new channel. Use channelID of 0 for creating channels. */
    CHECK_ERROR(ts3server_setChannelVariableAsString(server_id, 0, CHANNEL_NAME, channel_name));
    CHECK_ERROR(ts3server_setChannelVariableAsInt(server_id, 0, CHANNEL_FLAG_PERMANENT, 1));
    CHECK_ERROR(ts3server_setChannelVariableAsInt(server_id, 0, CHANNEL_FLAG_SEMI_PERMANENT, 0));

    /* Flush changes to server */
    CHECK_ERROR(ts3server_flushChannelCreation(server_id, channel_parent_id, &channel_id));

    printf("Created channel: %llu\n", channel_id);
    return channel_id;
}

int main()
{
    /* Create struct for callback function pointers */
    struct ServerLibFunctions funcs;

    /* Initialize all callbacks with NULL */
    memset(&funcs, 0, sizeof(struct ServerLibFunctions));

    /* Initialize server lib with callbacks */
    CHECK_ERROR_FATAL(ts3server_initServerLib(&funcs, LogType_CONSOLE, NULL));

    /* Create virtual server using default port 9987 with max 10 slots */
    /* listen on any address on ipv4 and ipv6 (it is also possible to enter multiple ipv4 and ipv6 addresses here) */
    uint64 server_id;
    CHECK_ERROR_FATAL(ts3server_createVirtualServer(9987, "0.0.0.0, ::", "TeamSpeak SDK Testserver", "", 10, &server_id));

    {   // create some channels
        const int channel_count = 5;
        for (int i = 0; i < channel_count; ++i)
        {
            char channel_name[23];
            sprintf(channel_name, "%i", i);
            create_channel(server_id, channel_name);
        }
    }

    /* Wait for user input */
    printf("\n--- Enter [q] to shutdown server and exit ---\n");
    printf("\n--- Enter [w] to toggle all-channel whispering ---\n");
    
    while (is_terminating == false)
    {
        int c = getc(stdin);
        switch (c)
        {
            case 'q':
            {
                /* Stop virtual server */
                CHECK_ERROR_FATAL(ts3server_stopVirtualServer(server_id));
                is_terminating = true;
                break;
            }
            case 'w':
                toggle_whisper(server_id);
                break;
            default:
                sleep_ms(100);
                break;
        }
    }

    /* Shutdown server lib */
    CHECK_ERROR_FATAL(ts3server_destroyServerLib());

    return 0;
}
